// -*- Java -*-
/*!
 * @file  HandflagBoyImpl.java
 * @brief ModuleDescription
 * @date  $Date$
 *
 * $Id$
 */

import RTC.TimedLong;
import RTC.ReturnCode_t;
import jp.go.aist.rtm.RTC.util.DataRef;
import jp.go.aist.rtm.RTC.port.InPort;
import jp.go.aist.rtm.RTC.port.ConnectorListenerType;
import jp.go.aist.rtm.RTC.port.ConnectorDataListenerType;
import jp.go.aist.rtm.RTC.port.ConnectorDataListenerT;
import jp.go.aist.rtm.RTC.port.ConnectorDataListener;
import jp.go.aist.rtm.RTC.port.ConnectorBase;
import jp.go.aist.rtm.RTC.Manager;
import jp.go.aist.rtm.RTC.DataFlowComponentBase;

/*!
 * @class HandflagBoyImpl
 * @brief ModuleDescription
 *
 */
public class HandflagBoyImpl extends DataFlowComponentBase {
    HandflagBoyFrame hfb;
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
	public HandflagBoyImpl(Manager manager) {
        super(manager);
        // <rtc-template block="initializer">
        m_dp_Int_val = new TimedLong();
        m_dp_Int_Data = new DataRef<TimedLong>(m_dp_Int_val);
        m_dp_IntIn = new InPort<TimedLong>("dp_IntIn", m_dp_Int_Data);
        // </rtc-template>

    }

    /**
     *
     * The initialize action (on CREATED->ALIVE transition)
     * formaer rtc_init_entry() 
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onInitialize() {
        // Registration: InPort/OutPort/Service
        // <rtc-template block="registration">
        // Set InPort buffers
        addInPort("dp_IntIn", m_dp_IntIn);
        // </rtc-template>
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_BUFFER_WRITE,
                            new DataListener("ON_BUFFER_WRITE"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_BUFFER_FULL, 
                            new DataListener("ON_BUFFER_FULL"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_BUFFER_WRITE_TIMEOUT, 
                            new DataListener("ON_BUFFER_WRITE_TIMEOUT"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_BUFFER_OVERWRITE, 
                            new DataListener("ON_BUFFER_OVERWRITE"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_BUFFER_READ, 
                            new DataListener("ON_BUFFER_READ"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_SEND, 
                            new DataListener("ON_SEND"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_RECEIVED,
                            new DataListener("ON_RECEIVED"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_RECEIVER_FULL, 
                            new DataListener("ON_RECEIVER_FULL"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_RECEIVER_TIMEOUT, 
                            new DataListener("ON_RECEIVER_TIMEOUT"));
        m_dp_IntIn.addConnectorDataListener(
                            ConnectorDataListenerType.ON_RECEIVER_ERROR,
                            new DataListener("ON_RECEIVER_ERROR"));
        hfb = new HandflagBoyFrame();
        hfb.pack();
        hfb.setVisible(true);
        return super.onInitialize();
    }

    /***
     *
     * The finalize action (on ALIVE->END transition)
     * formaer rtc_exiting_entry()
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onFinalize() {
//        return super.onFinalize();
//    }

    /***
     *
     * The startup action when ExecutionContext startup
     * former rtc_starting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStartup(int ec_id) {
//        return super.onStartup(ec_id);
//    }

    /***
     *
     * The shutdown action when ExecutionContext stop
     * former rtc_stopping_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onShutdown(int ec_id) {
//        return super.onShutdown(ec_id);
//    }

    /***
     *
     * The activated action (Active state entry action)
     * former rtc_active_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onActivated(int ec_id) {
        return super.onActivated(ec_id);
    }

    /***
     *
     * The deactivated action (Active state exit action)
     * former rtc_active_exit()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onDeactivated(int ec_id) {
        return super.onDeactivated(ec_id);
    }

    /***
     *
     * The execution action that is invoked periodically
     * former rtc_active_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
    @Override
    protected ReturnCode_t onExecute(int ec_id) {
       if( m_dp_IntIn.isNew() ) {
            m_dp_IntIn.read();
       //   System.out.print( "Received: " + m_dp_Int_Data.v.data + " " );
       //   System.out.print( "TimeStamp: " + m_dp_Int_Data.v.tm.sec + "[s] " );
       //   System.out.println( m_dp_Int_Data.v.tm.nsec + "[ns]" );
            hfb.setState((int)m_dp_Int_Data.v.data); // 手旗少年のコントロール
       }
        try {
            Thread.sleep(1);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
       return super.onExecute(ec_id);
    }

    /***
     *
     * The aborting action when main logic error occurred.
     * former rtc_aborting_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//  @Override
//  public ReturnCode_t onAborting(int ec_id) {
//      return super.onAborting(ec_id);
//  }

    /***
     *
     * The error action in ERROR state
     * former rtc_error_do()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    public ReturnCode_t onError(int ec_id) {
//        return super.onError(ec_id);
//    }

    /***
     *
     * The reset action that is invoked resetting
     * This is same but different the former rtc_init_entry()
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onReset(int ec_id) {
//        return super.onReset(ec_id);
//    }

    /***
     *
     * The state update action that is invoked after onExecute() action
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onStateUpdate(int ec_id) {
//        return super.onStateUpdate(ec_id);
//    }

    /***
     *
     * The action that is invoked when execution context's rate is changed
     * no corresponding operation exists in OpenRTm-aist-0.2.0
     *
     * @param ec_id target ExecutionContext Id
     *
     * @return RTC::ReturnCode_t
     * 
     * 
     */
//    @Override
//    protected ReturnCode_t onRateChanged(int ec_id) {
//        return super.onRateChanged(ec_id);
//    }
//
    // DataInPort declaration
    // <rtc-template block="inport_declare">
    protected TimedLong m_dp_Int_val;
    protected DataRef<TimedLong> m_dp_Int_Data;
    /*!
     */
    protected InPort<TimedLong> m_dp_IntIn;

        // </rtc-template>

    // DataOutPort declaration
    // <rtc-template block="outport_declare">
    
    // </rtc-template>

    // CORBA Port declaration
    // <rtc-template block="corbaport_declare">
    
    // </rtc-template>

    // Service declaration
    // <rtc-template block="service_declare">
    
    // </rtc-template>

    // Consumer declaration
    // <rtc-template block="consumer_declare">
    
    // </rtc-template>
    class DataListener extends ConnectorDataListenerT<TimedLong>{
        public DataListener(final String name){
            super(TimedLong.class);
            m_name = name;
        }

        public void operator(final ConnectorBase.ConnectorInfo arg,
                               final TimedLong data) {
            ConnectorBase.ConnectorInfo info =(ConnectorBase.ConnectorInfo)arg;
        //  System.out.println("------------------------------");
        //  System.out.println("Listener:       "+m_name);
        //  System.out.println("Profile::name:  "+info.name);
        //  System.out.println("Profile::id:    "+info.id);
//            System.out.println("Profile::properties: ");
//            System.out.println(info.properties);
        //  System.out.println("Data:           "+data.data);
        //  System.out.println("------------------------------");
        }
        public String m_name;
    }
}
